/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of FRETSaw.
 *
 *    FRETSaw is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FRETSaw is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FretSaw.Display;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import gov.nih.info.rsb.imagej.JImagePanel;
import ij.ImagePlus;

import javax.swing.JLabel;
import javax.swing.JPanel;

import uk.ac.bbsrc.babraham.FretSaw.FretImageCreator;
import uk.ac.bbsrc.babraham.FretSaw.IO.ImageSet;

public class ImageSetPanel extends JPanel implements Runnable {

	private ImageSet images = null;
	private ImagePlus fretImage = null;
	private JPanel imagePanel;
	private JImagePanel [] panels;
	private double widestWidth = 0;
	private JPanel fretPanel;
	private boolean updating = false;
	private int currentFrame = 1;
	
	public void setImages (ImageSet images) {
		
		// Clean up any existing displays
		if (imagePanel != null) {
			removeAll();
		}
		validate();
		currentFrame = 1;
		widestWidth = images.channel1().getWidth();
		
		if (images != null) {
			images.closeAll();
		}
		
		this.images = images;

		// Make sure we're starting on frame 1
		images.channel1().setSlice(1);
		images.channel2().setSlice(2);
		
		widestWidth /=2;
		
		panels = new JImagePanel[3];
		
		imagePanel = new JPanel();
		
		imagePanel.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.weighty=0.5;
		gbc.weightx=0.5;
		gbc.fill=GridBagConstraints.BOTH;
		
		JPanel CFPanel = new JPanel();
		CFPanel.setLayout(new BorderLayout());
		CFPanel.add(new JLabel("CFP",JLabel.CENTER),BorderLayout.NORTH);
		panels[0] = new JImagePanel(images.channel1());
		CFPanel.add(panels[0],BorderLayout.CENTER);
		gbc.weightx=images.channel1().getWidth()/widestWidth;
		imagePanel.add(CFPanel,gbc);
		
		gbc.gridx++;
		JPanel YFPanel = new JPanel();
		YFPanel.setLayout(new BorderLayout());
		YFPanel.add(new JLabel("YFP",JLabel.CENTER),BorderLayout.NORTH);
		panels[1] = new JImagePanel(images.channel2());
		YFPanel.add(panels[1],BorderLayout.CENTER);
		gbc.weightx=images.channel1().getWidth()/widestWidth;
		imagePanel.add(YFPanel,gbc);
	
		gbc.gridx++;
		fretPanel = new JPanel();
		fretPanel.setLayout(new BorderLayout());
		fretPanel.add(new LoadingPanel(),BorderLayout.CENTER);
//		// 30px is the extra width for the scale bar
		gbc.weightx=(images.channel1().getWidth()+30)/widestWidth;
		imagePanel.add(fretPanel,gbc);

		
		setLayout(new BorderLayout());
		add(imagePanel,BorderLayout.CENTER);
		
		add(new FrameChangeBar(this),BorderLayout.SOUTH);
		validate();
		update();
	}
	
	public void update () {
		Thread t = new Thread(this);
		t.start();
	}
	
	public int frameCount () {
		return images.channel1().getStackSize();
	}
	
	public void setFrame (int frame) {
		currentFrame = frame;
		images.channel1().setSlice(frame);
		images.channel2().setSlice(frame);
		if (fretImage != null) fretImage.setSlice(frame);
		
		for (int i=0;i<panels.length;i++) {
			if (panels[i] != null) {
				panels[i].setImageUpdated();
				panels[i].repaint();
			}
		}
		
	}
	
	public ImagePlus getFretImage () {
		return fretImage;
	}

	public void run() {

		if (updating) return;
		updating = true;
		
		if (fretImage != null) {
			fretImage.close();
		}
		fretPanel.removeAll();
		panels[2] = null;
		fretPanel.add(new LoadingPanel(),BorderLayout.CENTER);
		fretPanel.validate();
		
		fretImage = FretImageCreator.createFretImage(images.channel1(), images.channel2());
		
		fretPanel.removeAll();
		fretPanel.add(new JLabel("FRET",JLabel.CENTER),BorderLayout.NORTH);
		panels[2] = new JImagePanel(fretImage);
		fretPanel.add(panels[2],BorderLayout.CENTER);
	
		fretPanel.validate();
		validate();
		setFrame(currentFrame);
		updating = false;
	}
	
}
